@@ -173,7 +173,7 @@ capture_online(const char *dev, const char *outfile)
     capinfo->ip_reasm = vector_create(0, 10);
 
     // Add this capture information as packet source
-    vector_append(capture_cfg.sources, capinfo);
+    capture_add_source(capinfo);
 
     // If requested store packets in a dump file
     if (outfile && !capture_cfg.pd) {
@@ -238,7 +238,7 @@ capture_offline(const char *infile, const char *outfile)
     capinfo->ip_reasm = vector_create(0, 10);
 
     // Add this capture information as packet source
-    vector_append(capture_cfg.sources, capinfo);
+    capture_add_source(capinfo);
 
     // If requested store packets in a dump file
     if (outfile && !capture_cfg.pd) {
@@ -448,6 +448,9 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
     uint32_t len_data = 0;
     //! Link + Extra header size
     uint16_t link_hl = capinfo->link_hl;
+#ifdef USE_IPV6
+    struct ip6_frag *ip6f;
+#endif
 
     // Skip VLAN header if present
     if (capinfo->link == DLT_EN10MB) {
@@ -485,57 +488,70 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
         }
     }
 
-    // Get IP header
-    ip4 = (struct ip *) (packet + link_hl);
+    while (*size >= sizeof(struct ip)) {
+        // Get IP header
+        ip4 = (struct ip *) (packet + link_hl);
 
 #ifdef USE_IPV6
-    // Get IPv6 header
-    ip6 = (struct ip6_hdr *) (packet + link_hl);
+        // Get IPv6 header
+        ip6 = (struct ip6_hdr *) (packet + link_hl);
 #endif
 
-    // Get IP version
-    ip_ver = ip4->ip_v;
+        // Get IP version
+        ip_ver = ip4->ip_v;
 
-    switch (ip_ver) {
-        case 4:
-            ip_hl = ip4->ip_hl * 4;
-            ip_proto = ip4->ip_p;
-            ip_off = ntohs(ip4->ip_off);
-            ip_len = ntohs(ip4->ip_len);
+        switch (ip_ver) {
+            case 4:
+                ip_hl = ip4->ip_hl * 4;
+                ip_proto = ip4->ip_p;
+                ip_off = ntohs(ip4->ip_off);
+                ip_len = ntohs(ip4->ip_len);
 
-            ip_frag = ip_off & (IP_MF | IP_OFFMASK);
-            ip_frag_off = (ip_frag) ? (ip_off & IP_OFFMASK) * 8 : 0;
-            ip_id = ntohs(ip4->ip_id);
+                ip_frag = ip_off & (IP_MF | IP_OFFMASK);
+                ip_frag_off = (ip_frag) ? (ip_off & IP_OFFMASK) * 8 : 0;
+                ip_id = ntohs(ip4->ip_id);
 
-            inet_ntop(AF_INET, &ip4->ip_src, src.ip, sizeof(src.ip));
-            inet_ntop(AF_INET, &ip4->ip_dst, dst.ip, sizeof(dst.ip));
-            break;
+                inet_ntop(AF_INET, &ip4->ip_src, src.ip, sizeof(src.ip));
+                inet_ntop(AF_INET, &ip4->ip_dst, dst.ip, sizeof(dst.ip));
+                break;
 #ifdef USE_IPV6
-        case 6:
-            ip_hl = sizeof(struct ip6_hdr);
-            ip_proto = ip6->ip6_nxt;
-            ip_len = ntohs(ip6->ip6_ctlun.ip6_un1.ip6_un1_plen) + ip_hl;
+            case 6:
+                ip_hl = sizeof(struct ip6_hdr);
+                ip_proto = ip6->ip6_nxt;
+                ip_len = ntohs(ip6->ip6_ctlun.ip6_un1.ip6_un1_plen) + ip_hl;
 
-            if (ip_proto == IPPROTO_FRAGMENT) {
-                struct ip6_frag *ip6f = (struct ip6_frag *) (ip6 + ip_hl);
-                ip_frag_off = ntohs(ip6f->ip6f_offlg & IP6F_OFF_MASK);
-                ip_id = ntohl(ip6f->ip6f_ident);
-            }
+                if (ip_proto == IPPROTO_FRAGMENT) {
+                    ip_frag = 1;
+                    ip6f = (struct ip6_frag *) (packet + link_hl + ip_hl);
+                    ip_frag_off = ntohs(ip6f->ip6f_offlg & IP6F_OFF_MASK);
+                    ip_id = ntohl(ip6f->ip6f_ident);
+                }
 
-            inet_ntop(AF_INET6, &ip6->ip6_src, src.ip, sizeof(src.ip));
-            inet_ntop(AF_INET6, &ip6->ip6_dst, dst.ip, sizeof(dst.ip));
-            break;
+                inet_ntop(AF_INET6, &ip6->ip6_src, src.ip, sizeof(src.ip));
+                inet_ntop(AF_INET6, &ip6->ip6_dst, dst.ip, sizeof(dst.ip));
+                break;
 #endif
-        default:
-            return NULL;
+            default:
+                return NULL;
+        }
+
+        // Fixup VSS trailer in ethernet packets
+        *caplen = link_hl + ip_len;
+
+        // Remove IP Header length from payload
+        *size = *caplen - link_hl - ip_hl;
+
+        if (ip_proto == IPPROTO_IPIP) {
+            // The payload is an incapsulated IP packet (IP-IP tunnel)
+            // so we simply skip the "outer" IP header and repeat.
+            // NOTE: this will break IP reassembly if the "outer"
+            // packet is fragmented.
+            link_hl += ip_hl;
+        } else {
+            break;
+        }
     }
 
-    // Fixup VSS trailer in ethernet packets
-    *caplen = link_hl + ip_len;
-
-    // Remove IP Header length from payload
-    *size = *caplen - link_hl - ip_hl;
-
     // If no fragmentation
     if (ip_frag == 0) {
         // Just create a new packet with given network data
@@ -566,14 +582,25 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
 
     // Add this IP content length to the total captured of the packet
     pkt->ip_cap_len += ip_len - ip_hl;
+#ifdef USE_IPV6
+    if (ip_ver == 6 && ip_frag) {
+        pkt->ip_cap_len -= sizeof(struct ip6_frag);
+    }
+#endif
 
     // Calculate how much data we need to complete this packet
     // The total packet size can only be known using the last fragment of the packet
     // where 'No more fragments is enabled' and it's calculated based on the
     // last fragment offset
-    if ((ip_off & IP_MF) == 0) {
+    if (ip_ver == 4 && (ip_off & IP_MF) == 0) {
         pkt->ip_exp_len = ip_frag_off + ip_len - ip_hl;
     }
+#ifdef USE_IPV6
+    if (ip_ver == 6 && ip_frag && (ip6f->ip6f_offlg & htons(0x01)) == 0) {
+        pkt->ip_exp_len = ip_frag_off + ip_len - ip_hl - sizeof(struct ip6_frag);
+    }
+#endif
+
 
     // If we have the whole packet (captured length is expected length)
     if (pkt->ip_cap_len == pkt->ip_exp_len) {
@@ -581,8 +608,22 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
         // Calculate assembled IP payload data
         it = vector_iterator(pkt->frames);
         while ((frame = vector_iterator_next(&it))) {
-            struct ip *frame_ip = (struct ip *) (frame->data + link_hl);
-            len_data += ntohs(frame_ip->ip_len) - frame_ip->ip_hl * 4;
+            switch (ip_ver) {
+                case 4: {
+                    struct ip *frame_ip = (struct ip *) (frame->data + link_hl);
+                    len_data += ntohs(frame_ip->ip_len) - frame_ip->ip_hl * 4;
+                    break;
+                }
+#ifdef USE_IPV6
+                case 6: {
+                    struct ip6_hdr *frame_ip6 = (struct ip6_hdr *) (frame->data + link_hl);
+                    len_data += ntohs(frame_ip6->ip6_ctlun.ip6_un1.ip6_un1_plen);
+                    break;
+                }
+#endif
+                default:
+                    break;
+            }
         }
 
         // Check packet content length
@@ -594,14 +635,39 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
 
         it = vector_iterator(pkt->frames);
         while ((frame = vector_iterator_next(&it))) {
-            // Get IP header
-            struct ip *frame_ip = (struct ip *) (frame->data + link_hl);
-            memcpy(packet + link_hl + ip_hl + (ntohs(frame_ip->ip_off) & IP_OFFMASK) * 8,
-                   frame->data + link_hl + frame_ip->ip_hl * 4,
-                   ntohs(frame_ip->ip_len) - frame_ip->ip_hl * 4);
+            switch (ip_ver) {
+                case 4: {
+                    // Get IP header
+                    struct ip *frame_ip = (struct ip *) (frame->data + link_hl);
+                    memcpy(packet + link_hl + ip_hl + (ntohs(frame_ip->ip_off) & IP_OFFMASK) * 8,
+                           frame->data + link_hl + frame_ip->ip_hl * 4,
+                           ntohs(frame_ip->ip_len) - frame_ip->ip_hl * 4);
+
+                }
+                    break;
+#ifdef USE_IPV6
+                case 6: {
+                    struct ip6_hdr *frame_ip6 = (struct ip6_hdr*)(frame->data + link_hl);
+                    struct ip6_frag *frame_ip6f = (struct ip6_frag *)(frame->data + link_hl + ip_hl);
+                    uint16_t frame_ip_frag_off = ntohs(frame_ip6f->ip6f_offlg & IP6F_OFF_MASK);
+                    memcpy(packet + link_hl + ip_hl + sizeof(struct ip6_frag) + frame_ip_frag_off,
+                            frame->data + link_hl + ip_hl + sizeof (struct ip6_frag),
+                            ntohs(frame_ip6->ip6_ctlun.ip6_un1.ip6_un1_plen));
+                    pkt->proto = frame_ip6f->ip6f_nxt;
+                }
+                    break;
+#endif
+                default:
+                    break;
+            }
         }
 
         *caplen = link_hl + ip_hl + len_data;
+#ifdef USE_IPV6
+        if (ip_ver == 6) {
+            *caplen += sizeof(struct ip6_frag);
+        }
+#endif
         *size = len_data;
 
         // Return the assembled IP packet
@@ -1074,6 +1140,12 @@ capture_tls_server()
     return capture_cfg.tlsserver;
 }
 
+void
+capture_add_source(struct capture_info *capinfo)
+{
+    vector_append(capture_cfg.sources, capinfo);
+}
+
 int
 capture_sources_count()
 {
